// Rules parsing
// antecedent
function getAntecedent(rule) {
    let withoutIf = rule.split('if');
    return withoutIf[1].trim().split('then')[0].trim();
}

function getRuleComponents(antecedent) {
    return antecedent.split('and').map((i) => i.trim());
}

// consequent
function getConsequent(rule) {
    let withoutIf = rule.split('if');
    return withoutIf[1].trim().split('then')[1].trim();
}

// common
function getVariable(variableComponents) {
    return variableComponents.split('is')[0].trim();
}

function getVariableValue(variableComponents) {
    return variableComponents.split('is')[1].trim();
}

// Rules creation

/* exported MessageTypesEnum */
let MessageTypesEnum = {
    INFO: "info",
    SUCCESS: "success",
    WARNING: "warning",
    DANGER: "danger"
};
Object.freeze(MessageTypesEnum);

function isEmpty(value) {
    if (typeof value === "function") {
        return false;
    }
    return (value == null || value.length === 0);
}

/* exported showFeedbackMessage */
function showFeedbackMessage(message, type) {
    alert(type + ' ' + message);
}

/* exported errorHandler */
function errorHandler(response, callBack, errorCallBack) {
    if (!isEmpty(response.error)) {
        if (!isEmpty(errorCallBack)) {
            errorCallBack(response.data);
        }
        throw response.error.code + ": " + response.error.message +
        " / Details: " + response.error.data;
    }
    if (!isEmpty(callBack)) {
        callBack(response);
    }
}

/* exported getFromRest */
async function getFromRest(url) {
    return await $.ajax({
        url: url,
        method: 'get',
        cache: true,
        dataType: 'json'
    });
}

/* exported createRule */
function createRule() {
    let ruleString = "if ";
    let inp = $('.selectpicker.inputVar').children(':selected').map(function () {
        return $(this).text();
    }).get();
    let inpVal = $('.selectpicker.inputVal').children(':selected').map(function () {
        return $(this).text();
    }).get();
    let out = $('.selectpicker.outVar').children(':selected').map(function () {
        return $(this).text();
    }).get();
    let outVal = $('.selectpicker.outVal').children(':selected').map(function () {
        return $(this).text();
    }).get();
    for (let i = 0; i < inp.length; i++) {
        if (i > 0) {
            ruleString += ' and ';
        }
        ruleString += inp[i] + " is " + inpVal[i];
    }
    ruleString += " then ";
    for (let i = 0; i < out.length; i++) {
        if (i > 0) {
            ruleString += ' and ';
        }
        ruleString += out[i] + " is " + outVal[i];
    }
    $('#ruleContent').val(ruleString);
}

/* exported fillSelect */
function fillSelect(selectElement, values, selectedVal) {
    $(selectElement).html("");
    $.each(values, function (key, value) {
        $(selectElement).append($("<option />").val(value.id).text(value.name));
    });
    $(selectElement).children().each(function () {
        if ($(this).text() === selectedVal) {
            $(this).prop('selected', true);
        }
    });
}

async function fillFuzzyTerms(variablesElement, fuzzyTermsElement, termVal) {
    let fuzzyTerms = await getFromRest("/rule/getFuzzyTerms/" + $(variablesElement).val());
    let fuzzyTermsData = [];
    $.each(fuzzyTerms, function (key, value) {
        fuzzyTermsData.push({
            id: value.id,
            name: value.description
        });
    });
    fillSelect(fuzzyTermsElement, fuzzyTermsData, termVal);
}

async function fillVariables(projectId, variablesElement, variableVal, cls) {
    let url = '/rule/';
    if (cls === 'inputVar') {
        url += 'getInputVariables/';
    } else {
        url += 'getOutputVariables/';
    }
    let variables = await getFromRest(url + projectId);
    let variablesData = [];
    $.each(variables, function (key, value) {
        variablesData.push({
            id: value.id,
            name: value.name
        });
    });
    fillSelect(variablesElement, variablesData, variableVal);
}

async function variableValueChanged(variablesElement, fuzzyTermsElement) {
    await fillFuzzyTerms(variablesElement, fuzzyTermsElement);
    $(".selectpicker").selectpicker("refresh");
    createRule();
}

function fuzzyTermsValueChanged() {
    $(".selectpicker").selectpicker("refresh");
    createRule();
}

async function createVariableSelect(cls, projectId, variableVal) {
    let variablesElement = $("<select class='selectpicker " + cls + " m-2' data-live-search='true data-width='70%'></select>");
    await fillVariables(projectId, variablesElement, variableVal, cls);
    return variablesElement;
}

async function createFuzzyTermsSelect(cls, variablesElement, termVal) {
    let fuzzyTermsElement = $("<select class='selectpicker " + cls + " m-2' data-live-search='true data-width='70%'></select>");
    if ($(variablesElement).val()) {
        await fillFuzzyTerms(variablesElement, fuzzyTermsElement, termVal);
    }
    return fuzzyTermsElement;
}

function removeAntecedent(buttonElement) {
    $(buttonElement).parent().remove();
    fuzzyTermsValueChanged();
}

function removeConsequent(buttonElement) {
    $(buttonElement).parent().remove();
    fuzzyTermsValueChanged();
}

async function addAntecedent(parentElement, projectId, variableVal, termVal) {
    let rowElement = $("<div class='row'></div>");
    if ($(parentElement).find('.row').length) {
        $(rowElement).append("<label class='col col-md-1 m-2'>И</label>");
    } else {
        $(rowElement).append("<label class='col col-md-1 m-2'> </label>");
    }
    let variablesElement = await createVariableSelect('inputVar', projectId, variableVal);
    let fuzzyTermsElement = await createFuzzyTermsSelect('inputVal', variablesElement, termVal);
    $(variablesElement).on("change", function () {
        variableValueChanged(variablesElement, fuzzyTermsElement)
    });
    $(fuzzyTermsElement).on("change", function () {
        fuzzyTermsValueChanged()
    });
    $(rowElement).append(variablesElement);
    $(rowElement).append("<label class='col col-md-1  m-2'>есть</label>");
    $(rowElement).append(fuzzyTermsElement);
    if ($(parentElement).find('.row').length) {
        $(rowElement).append("<a href='#' class='btn btn-outline-dark m-2' onclick='removeAntecedent($(this))'>-</a>");
    }
    $(parentElement).append(rowElement);
}

async function addConsequent(parentElement, projectId, variableVal, termVal) {
    let rowElement = $("<div class='row'></div>");
    if ($(parentElement).find('.row').length) {
        $(rowElement).append("<label class='col col-md-1 m-2'>И</label>");
    } else {
        $(rowElement).append("<label class='col col-md-1 m-2'> </label>");
    }
    let variablesElement = await createVariableSelect('outVar', projectId, variableVal);
    let fuzzyTermsElement = await createFuzzyTermsSelect('outVal', variablesElement, termVal);
    $(variablesElement).on("change", async function () {
        await variableValueChanged(variablesElement, fuzzyTermsElement)
    });
    $(fuzzyTermsElement).on("change", function () {
        fuzzyTermsValueChanged()
    });
    $(rowElement).append(variablesElement);
    $(rowElement).append("<label class='col col-md-1  m-2'>есть</label>");
    $(rowElement).append(fuzzyTermsElement);
    if ($(parentElement).find('.row').length) {
        $(rowElement).append("<a href='#' class='btn btn-outline-dark m-2' onclick='removeConsequent($(this))'>-</a>");
    }
    $(parentElement).append(rowElement);
}

async function addAntecedentFromRule(parentElement, projectId, ruleContent) {
    let antecedentComponents = getRuleComponents(getAntecedent(ruleContent));
    for (let i = 0; i < antecedentComponents.length; i++) {
        let a = antecedentComponents[i];
        await addAntecedent(parentElement, projectId, getVariable(a), getVariableValue(a));
    }
}

async function addConsequentFromRule(parentElement, projectId, ruleContent) {
    let consequentComponents = getRuleComponents(getConsequent(ruleContent));
    for (let i = 0; i < consequentComponents.length; i++) {
        let c = consequentComponents[i];
        await addConsequent(parentElement, projectId, getVariable(c), getVariableValue(c));
    }
}

async function initSelects() {
    let ruleContentEl = $('#ruleContent');
    let projectIdEl = $('#projectId');
    if ($(ruleContentEl).val()) {
        await addAntecedentFromRule($('#rulesAntecedent'), $(projectIdEl).val(), $(ruleContentEl).val());
        await addConsequentFromRule($('#rulesConsequent'), $(projectIdEl).val(), $(ruleContentEl).val());
    } else {
        await addAntecedent($('#rulesAntecedent'), $(projectIdEl).val());
        await addConsequent($('#rulesConsequent'), $(projectIdEl).val());
    }
    $(".selectpicker").selectpicker("refresh");
}